@@ -13,8 +13,6 @@
 import datetime
 import os
 import re
-import subprocess
-import sys
 import time
 
 import psutil
@@ -22,12 +20,11 @@ from psutil import BSD
 from psutil import FREEBSD
 from psutil import NETBSD
 from psutil import OPENBSD
-from psutil._compat import PY3
 from psutil.tests import get_test_subprocess
+from psutil.tests import HAS_BATTERY
 from psutil.tests import MEMORY_TOLERANCE
 from psutil.tests import reap_children
-from psutil.tests import retry_before_failing
-from psutil.tests import run_test_module_by_name
+from psutil.tests import retry_on_failure
 from psutil.tests import sh
 from psutil.tests import unittest
 from psutil.tests import which
@@ -74,7 +71,7 @@ def muse(field):
 # =====================================================================
 
 
-@unittest.skipUnless(BSD, "BSD only")
+@unittest.skipIf(not BSD, "BSD only")
 class BSDSpecificTestCase(unittest.TestCase):
     """Generic tests common to all BSD variants."""
 
@@ -86,12 +83,9 @@ class BSDSpecificTestCase(unittest.TestCase):
     def tearDownClass(cls):
         reap_children()
 
+    @unittest.skipIf(NETBSD, "-o lstart doesn't work on NETBSD")
     def test_process_create_time(self):
-        cmdline = "ps -o lstart -p %s" % self.pid
-        p = subprocess.Popen(cmdline, shell=1, stdout=subprocess.PIPE)
-        output = p.communicate()[0]
-        if PY3:
-            output = str(output, sys.stdout.encoding)
+        output = sh("ps -o lstart -p %s" % self.pid)
         start_ps = output.replace('STARTED', '').strip()
         start_psutil = psutil.Process(self.pid).create_time()
         start_psutil = time.strftime("%a %b %e %H:%M:%S %Y",
@@ -143,8 +137,9 @@ class BSDSpecificTestCase(unittest.TestCase):
                 pass
             else:
                 self.assertEqual(stats.isup, 'RUNNING' in out, msg=out)
-                self.assertEqual(stats.mtu,
-                                 int(re.findall('mtu (\d+)', out)[0]))
+                if "mtu" in out:
+                    self.assertEqual(stats.mtu,
+                                     int(re.findall(r'mtu (\d+)', out)[0]))
 
 
 # =====================================================================
@@ -152,7 +147,7 @@ class BSDSpecificTestCase(unittest.TestCase):
 # =====================================================================
 
 
-@unittest.skipUnless(FREEBSD, "FREEBSD only")
+@unittest.skipIf(not FREEBSD, "FREEBSD only")
 class FreeBSDSpecificTestCase(unittest.TestCase):
 
     @classmethod
@@ -163,7 +158,20 @@ class FreeBSDSpecificTestCase(unittest.TestCase):
     def tearDownClass(cls):
         reap_children()
 
-    @retry_before_failing()
+    @staticmethod
+    def parse_swapinfo():
+        # the last line is always the total
+        output = sh("swapinfo -k").splitlines()[-1]
+        parts = re.split(r'\s+', output)
+
+        if not parts:
+            raise ValueError("Can't parse swapinfo: %s" % output)
+
+        # the size is in 1k units, so multiply by 1024
+        total, used, free = (int(p) * 1024 for p in parts[1:4])
+        return total, used, free
+
+    @retry_on_failure()
     def test_proc_memory_maps(self):
         out = sh('procstat -v %s' % self.pid)
         maps = psutil.Process(self.pid).memory_maps(grouped=False)
@@ -201,7 +209,7 @@ class FreeBSDSpecificTestCase(unittest.TestCase):
         self.assertEqual(gids.effective, int(egid))
         self.assertEqual(gids.saved, int(sgid))
 
-    @retry_before_failing()
+    @retry_on_failure()
     def test_proc_ctx_switches(self):
         tested = []
         out = sh('procstat -r %s' % self.pid)
@@ -221,7 +229,7 @@ class FreeBSDSpecificTestCase(unittest.TestCase):
         if len(tested) != 2:
             raise RuntimeError("couldn't find lines match in procstat out")
 
-    @retry_before_failing()
+    @retry_on_failure()
     def test_proc_cpu_times(self):
         tested = []
         out = sh('procstat -r %s' % self.pid)
@@ -241,39 +249,56 @@ class FreeBSDSpecificTestCase(unittest.TestCase):
         if len(tested) != 2:
             raise RuntimeError("couldn't find lines match in procstat out")
 
+    def test_cpu_frequency_against_sysctl(self):
+        # Currently only cpu 0 is frequency is supported in FreeBSD
+        # All other cores use the same frequency.
+        sensor = "dev.cpu.0.freq"
+        sysctl_result = int(sysctl(sensor))
+        self.assertEqual(psutil.cpu_freq().current, sysctl_result)
+
+        sensor = "dev.cpu.0.freq_levels"
+        sysctl_result = sysctl(sensor)
+        # sysctl returns a string of the format:
+        # <freq_level_1>/<voltage_level_1> <freq_level_2>/<voltage_level_2>...
+        # Ordered highest available to lowest available.
+        max_freq = int(sysctl_result.split()[0].split("/")[0])
+        min_freq = int(sysctl_result.split()[-1].split("/")[0])
+        self.assertEqual(psutil.cpu_freq().max, max_freq)
+        self.assertEqual(psutil.cpu_freq().min, min_freq)
+
     # --- virtual_memory(); tests against sysctl
 
-    @retry_before_failing()
+    @retry_on_failure()
     def test_vmem_active(self):
         syst = sysctl("vm.stats.vm.v_active_count") * PAGESIZE
         self.assertAlmostEqual(psutil.virtual_memory().active, syst,
                                delta=MEMORY_TOLERANCE)
 
-    @retry_before_failing()
+    @retry_on_failure()
     def test_vmem_inactive(self):
         syst = sysctl("vm.stats.vm.v_inactive_count") * PAGESIZE
         self.assertAlmostEqual(psutil.virtual_memory().inactive, syst,
                                delta=MEMORY_TOLERANCE)
 
-    @retry_before_failing()
+    @retry_on_failure()
     def test_vmem_wired(self):
         syst = sysctl("vm.stats.vm.v_wire_count") * PAGESIZE
         self.assertAlmostEqual(psutil.virtual_memory().wired, syst,
                                delta=MEMORY_TOLERANCE)
 
-    @retry_before_failing()
+    @retry_on_failure()
     def test_vmem_cached(self):
         syst = sysctl("vm.stats.vm.v_cache_count") * PAGESIZE
         self.assertAlmostEqual(psutil.virtual_memory().cached, syst,
                                delta=MEMORY_TOLERANCE)
 
-    @retry_before_failing()
+    @retry_on_failure()
     def test_vmem_free(self):
         syst = sysctl("vm.stats.vm.v_free_count") * PAGESIZE
         self.assertAlmostEqual(psutil.virtual_memory().free, syst,
                                delta=MEMORY_TOLERANCE)
 
-    @retry_before_failing()
+    @retry_on_failure()
     def test_vmem_buffers(self):
         syst = sysctl("vfs.bufspace")
         self.assertAlmostEqual(psutil.virtual_memory().buffers, syst,
@@ -281,48 +306,48 @@ class FreeBSDSpecificTestCase(unittest.TestCase):
 
     # --- virtual_memory(); tests against muse
 
-    @unittest.skipUnless(MUSE_AVAILABLE, "muse not installed")
+    @unittest.skipIf(not MUSE_AVAILABLE, "muse not installed")
     def test_muse_vmem_total(self):
         num = muse('Total')
         self.assertEqual(psutil.virtual_memory().total, num)
 
-    @unittest.skipUnless(MUSE_AVAILABLE, "muse not installed")
-    @retry_before_failing()
+    @unittest.skipIf(not MUSE_AVAILABLE, "muse not installed")
+    @retry_on_failure()
     def test_muse_vmem_active(self):
         num = muse('Active')
         self.assertAlmostEqual(psutil.virtual_memory().active, num,
                                delta=MEMORY_TOLERANCE)
 
-    @unittest.skipUnless(MUSE_AVAILABLE, "muse not installed")
-    @retry_before_failing()
+    @unittest.skipIf(not MUSE_AVAILABLE, "muse not installed")
+    @retry_on_failure()
     def test_muse_vmem_inactive(self):
         num = muse('Inactive')
         self.assertAlmostEqual(psutil.virtual_memory().inactive, num,
                                delta=MEMORY_TOLERANCE)
 
-    @unittest.skipUnless(MUSE_AVAILABLE, "muse not installed")
-    @retry_before_failing()
+    @unittest.skipIf(not MUSE_AVAILABLE, "muse not installed")
+    @retry_on_failure()
     def test_muse_vmem_wired(self):
         num = muse('Wired')
         self.assertAlmostEqual(psutil.virtual_memory().wired, num,
                                delta=MEMORY_TOLERANCE)
 
-    @unittest.skipUnless(MUSE_AVAILABLE, "muse not installed")
-    @retry_before_failing()
+    @unittest.skipIf(not MUSE_AVAILABLE, "muse not installed")
+    @retry_on_failure()
     def test_muse_vmem_cached(self):
         num = muse('Cache')
         self.assertAlmostEqual(psutil.virtual_memory().cached, num,
                                delta=MEMORY_TOLERANCE)
 
-    @unittest.skipUnless(MUSE_AVAILABLE, "muse not installed")
-    @retry_before_failing()
+    @unittest.skipIf(not MUSE_AVAILABLE, "muse not installed")
+    @retry_on_failure()
     def test_muse_vmem_free(self):
         num = muse('Free')
         self.assertAlmostEqual(psutil.virtual_memory().free, num,
                                delta=MEMORY_TOLERANCE)
 
-    @unittest.skipUnless(MUSE_AVAILABLE, "muse not installed")
-    @retry_before_failing()
+    @unittest.skipIf(not MUSE_AVAILABLE, "muse not installed")
+    @retry_on_failure()
     def test_muse_vmem_buffers(self):
         num = muse('Buffer')
         self.assertAlmostEqual(psutil.virtual_memory().buffers, num,
@@ -348,6 +373,23 @@ class FreeBSDSpecificTestCase(unittest.TestCase):
     #    self.assertAlmostEqual(psutil.cpu_stats().traps,
     #                           sysctl('vm.stats.sys.v_trap'), delta=1000)
 
+    # --- swap memory
+
+    def test_swapmem_free(self):
+        total, used, free = self.parse_swapinfo()
+        self.assertAlmostEqual(
+            psutil.swap_memory().free, free, delta=MEMORY_TOLERANCE)
+
+    def test_swapmem_used(self):
+        total, used, free = self.parse_swapinfo()
+        self.assertAlmostEqual(
+            psutil.swap_memory().used, used, delta=MEMORY_TOLERANCE)
+
+    def test_swapmem_total(self):
+        total, used, free = self.parse_swapinfo()
+        self.assertAlmostEqual(
+            psutil.swap_memory().total, total, delta=MEMORY_TOLERANCE)
+
     # --- others
 
     def test_boot_time(self):
@@ -359,7 +401,7 @@ class FreeBSDSpecificTestCase(unittest.TestCase):
 
     # --- sensors_battery
 
-    @unittest.skipUnless(psutil.sensors_battery(), "no battery")
+    @unittest.skipIf(not HAS_BATTERY, "no battery")
     def test_sensors_battery(self):
         def secs2hours(secs):
             m, s = divmod(secs, 60)
@@ -378,6 +420,7 @@ class FreeBSDSpecificTestCase(unittest.TestCase):
         else:
             self.assertEqual(secs2hours(metrics.secsleft), remaining_time)
 
+    @unittest.skipIf(not HAS_BATTERY, "no battery")
     def test_sensors_battery_against_sysctl(self):
         self.assertEqual(psutil.sensors_battery().percent,
                          sysctl("hw.acpi.battery.life"))
@@ -389,13 +432,41 @@ class FreeBSDSpecificTestCase(unittest.TestCase):
         else:
             self.assertEqual(secsleft, sysctl("hw.acpi.battery.time") * 60)
 
+    @unittest.skipIf(HAS_BATTERY, "has battery")
+    def test_sensors_battery_no_battery(self):
+        # If no battery is present one of these calls is supposed
+        # to fail, see:
+        # https://github.com/giampaolo/psutil/issues/1074
+        with self.assertRaises(RuntimeError):
+            sysctl("hw.acpi.battery.life")
+            sysctl("hw.acpi.battery.time")
+            sysctl("hw.acpi.acline")
+        self.assertIsNone(psutil.sensors_battery())
+
+    # --- sensors_temperatures
+
+    def test_sensors_temperatures_against_sysctl(self):
+        num_cpus = psutil.cpu_count(True)
+        for cpu in range(num_cpus):
+            sensor = "dev.cpu.%s.temperature" % cpu
+            # sysctl returns a string in the format 46.0C
+            sysctl_result = int(float(sysctl(sensor)[:-1]))
+            self.assertAlmostEqual(
+                psutil.sensors_temperatures()["coretemp"][cpu].current,
+                sysctl_result, delta=10)
+
+            sensor = "dev.cpu.%s.coretemp.tjmax" % cpu
+            sysctl_result = int(float(sysctl(sensor)[:-1]))
+            self.assertEqual(
+                psutil.sensors_temperatures()["coretemp"][cpu].high,
+                sysctl_result)
 
 # =====================================================================
 # --- OpenBSD
 # =====================================================================
 
 
-@unittest.skipUnless(OPENBSD, "OPENBSD only")
+@unittest.skipIf(not OPENBSD, "OPENBSD only")
 class OpenBSDSpecificTestCase(unittest.TestCase):
 
     def test_boot_time(self):
@@ -410,10 +481,11 @@ class OpenBSDSpecificTestCase(unittest.TestCase):
 # =====================================================================
 
 
-@unittest.skipUnless(NETBSD, "NETBSD only")
+@unittest.skipIf(not NETBSD, "NETBSD only")
 class NetBSDSpecificTestCase(unittest.TestCase):
 
-    def parse_meminfo(self, look_for):
+    @staticmethod
+    def parse_meminfo(look_for):
         with open('/proc/meminfo', 'rb') as f:
             for line in f:
                 if line.startswith(look_for):
@@ -477,4 +549,5 @@ class NetBSDSpecificTestCase(unittest.TestCase):
 
 
 if __name__ == '__main__':
-    run_test_module_by_name(__file__)
+    from psutil.tests.runner import run
+    run(__file__)
